"use strict";

/* exported logger */

var AddOverlayDialog = require('../AddOverlayDialog');
var Panel = require('../Panel');
var PanelControlElementType = require('../PanelControlElementType');

var GuiConnector = require('../../GuiConnector');
var logger = require('../../logger');
var Functions = require('../../Functions');
var NetworkError = require('../../NetworkError');

var HttpStatus = require('http-status-codes');

function OverlayPanel(params) {
  params.panelName = "overlays";
  params.scrollable = true;
  params.helpTip = "<p>Overlays tab allows to display or generate custom coloring of elements and interactions in the map.</p>"
      + "<p>General overlays are overlays accessible for every user viewing the content.</p>"
      + "<p>Custom overlays are user-provided overlays, this menu becomes available upon login (see below).</p>";
  Panel.call(this, params);

  var self = this;

  self._createOverlayPanelGui();

  var addButton = this.getControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON);

  addButton.onclick = function() {
    self.openAddOverlayDialog();
  };

  var titleElement = this.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE);
  self.setCustomOverlaysMessage(titleElement.innerHTML);

}

OverlayPanel.prototype = Object.create(Panel.prototype);
OverlayPanel.prototype.constructor = OverlayPanel;

OverlayPanel.prototype._createOverlayPanelGui = function() {
  var generalOverlaysDiv = Functions.createElement({
    type : "div",
    name : "generalOverlays",
    className : "searchPanel"
  });
  this.getElement().appendChild(generalOverlaysDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_DIV, generalOverlaysDiv);

  var generalOverlaysTitle = Functions.createElement({
    type : "h5",
    content : "GENERAL OVERLAYS:"
  });
  generalOverlaysDiv.appendChild(generalOverlaysTitle);

  var generalOverlaysTableDiv = Functions.createElement({
    type : "table",
    name : "generalOverlaysTab",
    className : "table table-bordered",
    style : "width:100%"
  });
  generalOverlaysDiv.appendChild(generalOverlaysTableDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE, generalOverlaysTableDiv);

  var customOverlaysDiv = Functions.createElement({
    type : "div",
    name : "customOverlays",
    className : "searchPanel"
  });
  this.getElement().appendChild(customOverlaysDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_DIV, customOverlaysDiv);

  var customOverlaysTitle = Functions.createElement({
    type : "h5",
    name : "customOverlaysTitle",
    content : "USER-PROVIDED OVERLAYS:"
  });
  customOverlaysDiv.appendChild(customOverlaysTitle);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE, customOverlaysTitle);

  var customOverlaysTableDiv = Functions.createElement({
    type : "table",
    name : "customOverlaysTab",
    className : "table table-bordered",
    style : "width:100%"
  });
  customOverlaysDiv.appendChild(customOverlaysTableDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE, customOverlaysTableDiv);

  var centerTag = Functions.createElement({
    type : "center"
  });
  customOverlaysDiv.appendChild(centerTag);

  var addOverlayButton = Functions.createElement({
    type : "button",
    name : "addOverlay",
    content : "Add overlay"
  });
  centerTag.appendChild(addOverlayButton);
  this.setControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON, addOverlayButton);
};

OverlayPanel.prototype.clear = function() {
  var table = this.getControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE);
  while (table.firstChild) {
    table.removeChild(table.firstChild);
  }

  table = this.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE);
  while (table.firstChild) {
    table.removeChild(table.firstChild);
  }

};

OverlayPanel.prototype.createTableHeader = function(edit) {
  var result = document.createElement("thead");

  var row = document.createElement("tr");

  var nameTd = document.createElement("th");
  nameTd.innerHTML = "Name";
  row.appendChild(nameTd);

  var viewTd = document.createElement("th");
  viewTd.innerHTML = "View";
  row.appendChild(viewTd);

  var dataTd = document.createElement("th");
  dataTd.innerHTML = "Data";
  row.appendChild(dataTd);

  if (edit) {
    var editTd = document.createElement("th");
    editTd.innerHTML = "Edit";
    row.appendChild(editTd);
  }

  result.appendChild(row);
  return result;
};

OverlayPanel.prototype.createOverlayRow = function(overlay, checked) {
  var self = this;
  var guiUtils = self.getGuiUtils();
  var result = document.createElement("tr");

  if (checked) {
    result.className = "active";
  }

  var nameTd = document.createElement("td");
  nameTd.innerHTML = overlay.getName();
  result.appendChild(nameTd);

  var viewTd = document.createElement("td");
  if (overlay.getInputDataAvailable()) {
    var checkbox = document.createElement("input");
    checkbox.type = "checkbox";
    checkbox.checked = checked;
    checkbox.onclick = function() {
      if (this.checked) {
        return self.getMap().openDataOverlay(overlay.getId()).then(null, GuiConnector.alert);
      } else {
        return self.getMap().removeSelectedLayout(overlay.getId()).then(null, GuiConnector.alert);
      }
    };
    viewTd.appendChild(checkbox);
  } else {
    var img = guiUtils.createIcon("icons/search.png");
    var link = document.createElement("a");
    link.href = "#";
    link.onclick = function() {
      self.getMap().openDataOverlay(overlay.getId());
      $(result).addClass('active').siblings().removeClass('active');
    };
    link.appendChild(img);
    viewTd.appendChild(link);
  }
  result.appendChild(viewTd);

  var dataTd = document.createElement("td");
  if (overlay.getInputDataAvailable()) {
    var button = document.createElement("button");
    button.setAttribute("name", "download-overlay-" + overlay.getId());
    button.onclick = function() {
      return ServerConnector.getOverlaySourceDownloadUrl({
        overlayId : overlay.getId()
      }).then(function(url) {
        return self.downloadFile(url);
      }).then(null, GuiConnector.alert);
    };
    button.innerHTML = "<span class='ui-icon ui-icon-arrowthickstop-1-s'></span>";
    dataTd.appendChild(button);
  }

  result.appendChild(dataTd);

  if (overlay.getCreator() !== "" && overlay.getCreator() !== undefined) {
    var editTd = document.createElement("td");
    var editButton = document.createElement("button");
    editButton.setAttribute("name", "editButton");
    editButton.onclick = function() {
      self.openEditOverlayDialog(overlay);
    };
    editButton.innerHTML = "<span class='ui-icon ui-icon-document'></span>";
    editTd.appendChild(editButton);
    result.appendChild(editTd);
  }
  result.title = overlay.getDescription();
  return result;
};

OverlayPanel.prototype.openEditOverlayDialog = function(overlay) {
  var self = this;
  var guiUtils = self.getGuiUtils();
  var content = document.createElement("fieldset");
  var nameInput = guiUtils.createInputText(overlay.getName());
  var row = guiUtils.createTableRow([ guiUtils.createLabel("Name: "), nameInput ]);
  content.appendChild(row);

  var descriptionInput = guiUtils.createTextArea(overlay.getDescription());
  row = guiUtils.createTableRow([ guiUtils.createLabel("Description: "), descriptionInput ]);
  content.appendChild(row);

  var buttons = [ {
    text : "SAVE",
    click : function() {
      var windowSelf = this;
      overlay.setName(nameInput.value);
      overlay.setDescription(descriptionInput.value);
      return ServerConnector.updateOverlay(overlay).then(function() {
        return self.refresh();
      }).then(function() {
        $(windowSelf).dialog("close");
      }).then(null, GuiConnector.alert);
    }
  }, {
    text : "REMOVE",
    click : function() {
      self.getMap().removeSelectedLayout(overlay.getId());

      var windowSelf = this;
      return ServerConnector.removeOverlay({
        overlayId : overlay.getId()
      }).then(function() {
        return self.refresh();
      }).then(function() {
        $(windowSelf).dialog("close");
      }).then(null, GuiConnector.alert);
    }
  }, {
    text : "CANCEL",
    click : function() {
      $(this).dialog("close");
    }
  } ];
  self.openDialog(content, {
    id : overlay.getId(),
    buttons : buttons,
    title : "Data overlay: " + overlay.getName(),
    className : "minerva-overlay-dialog",
  });
};

OverlayPanel.prototype.refresh = function() {
  var self = this;
  var user = null;

  var overlayTypes = [];
  var selectedOverlay = [];

  return ServerConnector.getOverlayTypes().then(function(types) {
    overlayTypes = types;
    return ServerConnector.getLoggedUser();
  }).then(function(loggedUser) {
    user = loggedUser;
    return self.getMap().getVisibleDataOverlays();
  }).then(function(visibleDataOverlays) {
    for (var j = 0; j < visibleDataOverlays.length; j++) {
      selectedOverlay[visibleDataOverlays[j].getId()] = true;
    }

    return ServerConnector.getOverlays({
      publicOverlay : false,
      creator : user.getLogin(),
    });
  }).then(
      function(customOverlays) {

        var id = self.getMap().getGoogleMap().getMapTypeId().substring(2);
        selectedOverlay[id] = true;

        self.clear();

        var generalOverlays = [];
        var overlay;

        var overlays = self.getMap().getLayouts();
        for (var i = 0; i < overlays.length; i++) {
          overlay = overlays[i];
          if (overlay.getCreator() === undefined || overlay.getCreator() === "") {
            generalOverlays.push(overlay);
          }
        }

        var table = self.getControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE);
        table.appendChild(self.createTableHeader());

        var body = document.createElement("tbody");
        table.appendChild(body);
        for (i = 0; i < generalOverlays.length; i++) {
          overlay = generalOverlays[i];
          body.appendChild(self.createOverlayRow(overlay, selectedOverlay[overlay.getId()]));
        }

        var title = self.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE);
        var addButton = self.getControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON);
        if (user.getLogin() === "anonymous") {
          title.innerHTML = 'YOU ARE NOT LOGGED IN. PLEASE, <a href="#">LOG IN</a>'
              + 'TO UPLOAD AND VIEW CUSTOM OVERLAYS<br/><center><button>LOGIN</button></center>';
          var openLoginDialog = function() {
            return self.getParent().getLoginDialog().open();
          };
          $(title).find("a")[0].onclick = openLoginDialog;
          $(title).find("button")[0].onclick = openLoginDialog;
          addButton.style.display = "none";
        } else {
          title.innerHTML = self.getCustomOverlaysMessage();
          addButton.style.display = "block";

          table = self.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE);
          table.appendChild(self.createTableHeader(true));

          body = document.createElement("tbody");
          table.appendChild(body);
          for (i = 0; i < customOverlays.length; i++) {
            overlay = customOverlays[i];
            body.appendChild(self.createOverlayRow(overlay, selectedOverlay[overlay.getId()]));
          }
        }

        self.onresize();
      });
};

OverlayPanel.prototype.setCustomOverlaysMessage = function(customOverlaysMessage) {
  this._customOverlaysMessage = customOverlaysMessage;
};

OverlayPanel.prototype.getCustomOverlaysMessage = function() {
  return this._customOverlaysMessage;
};


OverlayPanel.prototype.openAddOverlayDialog = function() {
  var self = this;
  var addOverlayDialog = new AddOverlayDialog({
    project : self.getProject(),
    customMap : self.getMap(),
    element : document.createElement("div"),
  });
  addOverlayDialog.addListener("onAddOverlay", function(e) {
    logger.info(e.arg);
    self.getMap().getModel().addLayout(e.arg);
    return self.refresh();
  })
  return addOverlayDialog.init().then(function() {
    return addOverlayDialog.open();
  });
};

OverlayPanel.prototype.init = function() {
  return this.refresh();
};
module.exports = OverlayPanel;
